#!/usr/bin/env python
# Copyright 2008 David D Lowe
# released under the GPLv2

from pigment import PigmentType, Pigment
import pigment
import os, sys
from glob import glob
if __name__ == "__main__" or 'pychecker' in sys.argv[0]:
    sys.path.append("..")
import handy
import ep_exceptions
import hashlib
from gettext import gettext as _


class GrubType(PigmentType):
    humanName = {"en":"Grub"}
    humanPluralName = {"en":"Grub"}
    codeName = "grub"
    abbreviation = "gr"
    directoryName = "grub"
    
    attachmentTypeHelp = _("The theme file should be a compressed xpm image file " +
        "with the extension xpm.gz")
        

class Grub(Pigment):
    
    
    def __init__(self):
        Pigment.__init__(self)
        self.type = GrubType()
        self.grubFilename = ""
        self.grubPath = ""
    
    def add_attachment(self, attachmentPath):
        if not len(self.attachments) == 0:
           raise(ep_exceptions.TooManyAttachmentsException(len(self.attachments)))
        if os.path.isabs(attachmentPath) and os.path.exists(attachmentPath):
            realPath = attachmentPath
        else:
            if os.path.exists(os.path.join("/boot/grub/splashimages", attachmentPath.split("/")[-1])):
                realPath = os.path.join("/boot/grub/splashimages", attachmentPath.split("/")[-1])
            else:
                self.grubPath = ""
                self.grubFilename = ""
                raise(ep_exceptions.AttachmentNotFoundException(attachmentPath))
        self.grubPath = realPath
        self.grubFilename = attachmentPath.split("/")[-1]
        Pigment.add_attachment(self, realPath)
    
    
    def install(self, installationDirectory=None, progressCallback=None, shell=None):
        exitPlease = False
        if progressCallback is None: progressCallback = lambda progress, query: False
        def change_progress_callback(progress, query):
            if progressCallback(progress / 2, query):
                exitPlease = True
                return True
            return False
        # copy pigment information over
        Pigment.install(self, installationDirectory, change_progress_callback, shell)
        if exitPlease: return
        
        # copy grub file
        if progressCallback is None: progressCallback = lambda progress, query: False
        if os.path.exists(os.path.join("/boot/grub/splashimages", self.grubFilename)):
            query = pigment.QUERY_OVERWRITE_DATA
        else:
            query = None
        if progressCallback(0.75, query): return
        shell.do(["mkdir", "-p", "/boot/grub/splashimages/"])
        shell.do(["cp", "-f", self.grubPath, "/boot/grub/splashimages/"])
        progressCallback(1.0, None)
            
    
    def remove(self, installationDirectory=None, progressCallback=None, shell=None):
        exitPlease = False
        def change_progress_callback(progress):
            if progressCallback(progress / 2):
                exitPlease = True
                return True
            return False
        # copy pigment information over
        Pigment.remove(self, installationDirectory, change_progress_callback, shell)
        if exitPlease: return
        
        # remove grub file
        if progressCallback is None: progressCallback = lambda(progress): False
        if progressCallback(0.75): return
        shell.do(["rm", "/boot/grub/splashimages/%s" % self.grubFilename])
        progressCallback(1.0)
    
    def activate(self, progressCallback=None, shell=None):
        if progressCallback is None:
            progressCallback = lambda(a1): False
        if progressCallback(0.0): return
        if os.path.exists("/boot/grub/splash.xpm.gz"):
            shell.do(["cp", "-f", "/boot/grub/splash.xpm.gz", "/boot/grub/splash.xpm.gz.epidermis.bak"])
            shell.do(["rm", "/boot/grub/splash.xpm.gz"])
        if progressCallback(0.25): return
        shell.do(["ln", "-s", "/boot/grub/splashimages/%s" % self.grubFilename, "/boot/grub/splash.xpm.gz"])
        if progressCallback(0.5): return
        shell.do(["update-grub"])
        progressCallback(1.0)
    
    def read(self, filePath):
        Pigment.read(self, filePath)
        
        from xml.dom import minidom
        doc = minidom.parse(filePath)
        root = doc.getElementsByTagName("pigment")[0]
        
        attachmentPath = filePath[:(-1*len(filePath.split("/")[-1]))]
        self.add_attachment(os.path.join(attachmentPath, "data", pigment.get_first_text( \
            root.getElementsByTagName("filename")[0])))
            
    def information_xml(self):
        """Returns the XML document describing the pigment"""
        doc = Pigment.information_xml(self)
        root = doc.getElementsByTagName("pigment")[0]
        
        
        xmlBit = doc.createElement("filename")
        tn = doc.createTextNode(self.grubFilename)
        xmlBit.appendChild(tn)
        root.appendChild(xmlBit)
        
        return doc
    
    
    def is_installed(self, installationDirectory=None):
        """Returns whether this pigment is installed
        installationDirectory: the directory in which to search for already 
        installed pigments"""
        if Pigment.is_installed(self, installationDirectory):
            if os.path.exists(os.path.join("/boot/grub/splashimages", self.grubFilename)):
                return True
        return False
    
    def is_activated(self):
        """Returns whether this pigment is applied. To check if the effects
        have been activated regardless of Epidermis' pigments, use is_meat_activated"""
        return self.is_meat_activated()

    def is_attachment_correct(self, attachmentIndex=None):
        if attachmentIndex is None and len(self.attachments) != 1:
            return False
        if not attachmentIndex is None:
            if attachmentIndex >= len(self.attachments):
                print >> sys.stderr, "no attachment with index ", attachmentIndex
            att = self.attachments[attachmentIndex]
        else:
            att = self.attachments[0]
        if not os.path.isfile(att):
            return False
        extension = att.split(".")[-2:]
        if not (extension[0], extension[1]) == ("xpm","gz"):
            return False
        return Pigment.is_attachment_correct(self, attachmentIndex)
    
    def convert_from_meat(self, meat, progressCallback=None):
        # TODO: convert_from_meat
        pass
    
    def is_meat_installed(self):
        """Returns whether the meat of this pigment or similar to it is installed"""
        (md1, md2) = (hashlib.md5(), hashlib.md5())
        try:
            f1 = file(self.grubPath, 'rb')
            f2 = file(os.path.join("/boot/grub/splashimages/", self.grubFilename), 'rb')
        except:
            return False
        for mm, ff in ((md1, f1), (md2, f2)):
            ss = ff.read(8096)
            if not ss:
                break
            mm.update(ss)
        return m1 == m2
        
    
    def is_meat_activated(self):
        """Returns whether the meat of this pigment or similar to it is activated"""
        # TODO: finish is_meat_activated
        (md1, md2) = (hashlib.md5(), hashlib.md5())
        try:
            f1 = file(self.grubPath, 'rb')
            f2 = file("/boot/grub/splash.xpm.gz", 'rb')
        except:
            return False
        for mm, ff in ((md1, f1), (md2, f2)):
            ss = ff.read(8096)
            if not ss:
                break
            mm.update(ss)
        return md1 == md2
    
    def does_activation_require_root(self):
        """Returns whether the activation requires root rights.
        If it does, the pigment object will need a pexpect bash shell as root
        to be passed to some of its methods"""
        return True
    
    def match_filter(self, string, lang="en"):
        """Returns whether this pigment should be a result for a search for the
        string passed"""
        for ii in (self.get_human_name(lang), self.get_description(lang)):
            if string.lower() in ii.lower():
                return True
        return False
    
    def get_display_data(self, index=0):
        """Returns the object of index to be added to a gtk.ListStore, this
        could be a string, a gtk.gdk.Pixbuf or a custom widget, it should
        be rendered using type.create_cell_renderer"""
        return


if __name__ == "__main__":
    main()