#!/usr/bin/env python

from pigment import PigmentType, Pigment
import pigment
import os, sys
from glob import glob
if __name__ == "__main__" or 'pychecker' in sys.argv[0]:
    sys.path.append("..")
from const import DEBUG
import ep_exceptions
import handy
import hashlib
from gettext import gettext as _


class UsplashType(PigmentType):
    humanName = {"en":"Usplash"}
    humanPluralName = {"en":"Usplashes"}
    codeName = "usplash"
    abbreviation = "uspl"
    directoryName = "usplashes"
    
    attachmentTypeHelp = _("A single file with the extension .so")
        

class Usplash(Pigment):
    
    
    def __init__(self):
        Pigment.__init__(self)
        self.type = UsplashType()
        self.usplashFilename = ""
        self.usplashPath = ""
    
    def add_attachment(self, attachmentPath):
        if not len(self.attachments) == 0:
           raise(ep_exceptions.TooManyAttachmentsException(len(self.attachments)))
        if os.path.isabs(attachmentPath) and os.path.exists(attachmentPath):
            realPath = attachmentPath
        else:
            if os.path.exists(os.path.join("/usr/lib/usplash", attachmentPath.split("/")[-1])):
                realPath = os.path.join("/usr/lib/usplash", attachmentPath.split("/")[-1])
            else:
                self.usplashPath = ""
                self.usplashFilename = ""
                raise(ep_exceptions.AttachmentNotFoundException(attachmentPath))
        self.usplashPath = realPath
        self.usplashFilename = attachmentPath.split("/")[-1]
        Pigment.add_attachment(self, realPath)
    
    
    def install(self, installationDirectory=None, progressCallback=None, shell=None):
        exitPlease = False
        if progressCallback is None: progressCallback = lambda progress, query: False
        def change_progress_callback(progress, query):
            if progressCallback(progress / 2, query):
                exitPlease = True
                return True
            return False
        # copy pigment information over
        Pigment.install(self, installationDirectory, change_progress_callback, shell)
        if exitPlease: return
        
        if progressCallback is None: progressCallback = lambda progress, query: False
        if os.path.exists("/usr/lib/usplash/%s" % self.usplashFilename):
            query = pigment.QUERY_OVERWRITE_DATA
        else:
            query = None
        if progressCallback(0.5, query): return
        shell.do(["cp", self.usplashPath, "/usr/lib/usplash/"])
        if progressCallback(0.75, None): return
        shell.do(["update-alternatives", "--install", "/usr/lib/usplash/usplash-artwork.so", \
            "usplash-artwork.so", "/usr/lib/usplash/%s" % self.usplashFilename, "10"])
        progressCallback(1.0, None)
            
    
    def remove(self, installationDirectory=None, progressCallback=None, shell=None):
        exitPlease = False
        def change_progress_callback(progress):
            if progressCallback(progress / 2):
                exitPlease = True
                return True
            return False
        # copy pigment information over
        Pigment.remove(self, installationDirectory, change_progress_callback, shell)
        if exitPlease: return
        
        if progressCallback is None: progressCallback = lambda(progress): False
        if progressCallback(0.75): return
        shell.do(["update-alternatives", "--remove", "usplash-artwork.so", "/usr/lib/usplash/%s" % self.usplashFilename])
        progressCallback(1.0)
    
    def activate(self, progressCallback=None, shell=None):
        if progressCallback is None:
            progressCallback = lambda(a1): False
        if progressCallback(0.0): return
        if DEBUG:
            shell.set_out_file(sys.stdout)
        shell.do(["update-alternatives", "--set", "usplash-artwork.so", "/usr/lib/usplash/%s" % self.usplashFilename])
        if progressCallback(0.25): return
        shell.do(["update-initramfs", "-u"])
        shell.do(["echo", "usplash done"])
        if DEBUG:
            shell.set_out_file(os.tmpfile())
        progressCallback(1.0)

    def read(self, filePath):
        Pigment.read(self, filePath)
        
        from xml.dom import minidom
        doc = minidom.parse(filePath)
        root = doc.getElementsByTagName("pigment")[0]
        
        attachmentPath = filePath[:(-1*len(filePath.split("/")[-1]))]
        self.add_attachment(os.path.join(attachmentPath, "data", pigment.get_first_text( \
            root.getElementsByTagName("filename")[0])))

    def information_xml(self):
        """Returns the XML document describing the pigment"""
        doc = Pigment.information_xml(self)
        root = doc.getElementsByTagName("pigment")[0]
        
        
        xmlBit = doc.createElement("filename")
        tn = doc.createTextNode(self.usplashFilename)
        xmlBit.appendChild(tn)
        root.appendChild(xmlBit)
        
        return doc
    
    
    def is_installed(self, installationDirectory=None):
        """Returns whether this pigment is installed
        installationDirectory: the directory in which to search for already 
        installed pigments"""
        if Pigment.is_installed(self, installationDirectory):
            if os.path.exists(os.path.join("/usr/lib/usplash/%s", self.usplashFilename)):
                return True
        return False
    
    def is_activated(self):
        """Returns whether this pigment is applied. To check if the effects
        have been activated regardless of Epidermis' pigments, use is_meat_activated"""
        return self.is_meat_activated()
    
    def is_attachment_correct(self, attachmentIndex=None):
        if attachmentIndex is None and len(self.attachments) != 1:
            return False
        if not attachmentIndex is None:
            if attachmentIndex >= len(self.attachments):
                print >> sys.stderr, "no attachment with index ", attachmentIndex
            att = self.attachments[attachmentIndex]
        else:
            att = self.attachments[0]
        if not os.path.isfile(att):
            return False
        if att.split(".")[-1] != "so":
            return False
        return Pigment.is_attachment_correct(self, attachmentIndex)
    
    def convert_from_meat(self, meat, progressCallback=None):
        # TODO: convert_from_meat
        pass
    
    def is_meat_installed(self):
        """Returns whether the meat of this pigment or similar to it is installed"""
        (md1, md2) = (hashlib.md5(), hashlib.md5())
        try:
            f1 = file(self.usplashPath, 'rb')
            f2 = file(os.path.join("/usr/lib/usplash/", self.usplashFilename), 'rb')
        except:
            return False
        for mm, ff in ((md1, f1), (md2, f2)):
            ss = ff.read(8096)
            if not ss:
                break
            mm.update(ss)
        return m1 == m2
        
    
    def is_meat_activated(self):
        """Returns whether the meat of this pigment or similar to it is activated"""
        # TODO: finish is_meat_activated
        (md1, md2) = (hashlib.md5(), hashlib.md5())
        try:
            f1 = file(self.usplashPath, 'rb')
            pop = os.popen(["update-alternatives", "--display", "usplash-artwork.so"])
            ss = ""
            for line in pop.readlines():
                line = line[:-1]
                if "link currently points to " in line:
                    ss = line.split("link currently points to ")[1]
                    break
            if ss == "": return False
            f2 = file(ss, 'rb')
        except:
            return False
        for mm, ff in ((md1, f1), (md2, f2)):
            ss = ff.read(8096)
            if not ss:
                break
            mm.update(ss)
        return m1 == m2
    
    def does_activation_require_root(self):
        """Returns whether the activation requires root rights.
        If it does, the pigment object will need a pexpect bash shell as root
        to be passed to some of its methods"""
        return True
    
    def match_filter(self, string, lang="en"):
        """Returns whether this pigment should be a result for a search for the
        string passed"""
        for ii in (self.get_human_name(lang), self.get_description(lang)):
            if string.lower() in ii.lower():
                return True
        return False
    
    def get_display_data(self, index=0):
        """Returns the object of index to be added to a gtk.ListStore, this
        could be a string, a gtk.gdk.Pixbuf or a custom widget, it should
        be rendered using type.create_cell_renderer"""
        return

def main():
    wp = GnomeSplash()
    print wp.information_xml().toxml()

if __name__ == "__main__":
    main()