#!/usr/bin/env python

from pigment import PigmentType, Pigment
import pigment
import sys, os
import gconf
if __name__ == "__main__" or 'pychecker' in sys.argv[0]:
    sys.path.append("..")
import ep_exceptions
import hashlib
from gettext import gettext as _


class WallpaperType(PigmentType):
    humanName = {"en":"Wallpaper"}
    humanPluralName = {"en":"Wallpapers"}
    codeName = "wallpaper"
    abbreviation = "wp"
    directoryName = "wallpapers"
    
    attachmentTypeHelp = _("The theme file should be an image file with the extension "+
        "png , jpg , gif or svg")
        

class Wallpaper(Pigment):
    
    
    def __init__(self):
        Pigment.__init__(self)
        self.type = WallpaperType()
        self.wallpaperFilename = ""
        self.wallpaperPath = ""
        self.pictureOptions = ""
        self.colourShadingType = ""
        self.primaryColour = ""
        self.secondaryColour = ""
    
    def add_attachment(self, attachmentPath):
        """Note: add_attachment will try and search for an existing alternative if
        attachmentPath doesn't exist"""
        if not len(self.attachments) == 0:
           raise(ep_exceptions.TooManyAttachmentsException(len(self.attachments)))
        if os.path.isabs(attachmentPath) and os.path.exists(attachmentPath):
            realPath = attachmentPath
        else:
            if os.path.exists(os.path.join("/usr/share/backgrounds/epidermis", self.codeName, attachmentPath.split("/")[-1])):
                realPath = os.path.join("/usr/share/backgrounds/epidermis",self.codeName, attachmentPath.split("/")[-1])
            else:
                self.wallpaperPath = ""
                self.wallpaperFilename = ""
                raise(ep_exceptions.AttachmentNotFoundException(attachmentPath))
        self.wallpaperPath = realPath
        self.wallpaperFilename = attachmentPath.split("/")[-1]
        Pigment.add_attachment(self, realPath)
    
    
    def install(self, installationDirectory=None, progressCallback=None, shell=None):
        exitPlease = False
        if progressCallback is None: progressCallback = lambda progress, query: False
        def change_progress_callback(progress, query):
            if progressCallback(progress / 2, None):
                exitPlease = True
                return True
            return False
        # copy pigment information over
        Pigment.install(self, installationDirectory, change_progress_callback, shell)
        #TODO: remove exitPlease
        if exitPlease: return
        
        # copy wallpaper file
        if progressCallback is None: progressCallback = lambda(progress, query): False
        if os.path.exists("/usr/share/backgrounds/epidermis/%s" % self.codeName):
            query = pigment.QUERY_OVERWRITE_DATA
        else:
            query = None
        if progressCallback(0.75, query): return
        shell.do(["mkdir", "-p", "/usr/share/backgrounds/epidermis/%s" % self.codeName])
        if not os.path.exists(self.wallpaperPath):
            raise(ep_exceptions.FileNotFoundException(self.wallpaperPath))
        
        shell.do(["cp", "-f", self.wallpaperPath, "/usr/share/backgrounds/epidermis/%s/" % self.codeName])
        shell.do(["echo", "hi"])
        progressCallback(1.0, None)
            
    
    def remove(self, installationDirectory=None, progressCallback=None, shell=None):
        exitPlease = False
        def change_progress_callback(progress):
            if progressCallback(progress / 2):
                exitPlease = True
                return True
            return False
        # copy pigment information over
        Pigment.remove(self, installationDirectory, change_progress_callback, shell)
        if exitPlease: return
        
        # remove wallpaper file
        #def ignore(progress): return False
        if progressCallback is None: progressCallback = lambda(progress): False
        if progressCallback(0.75): return
        shell.do(["rm", "-r", "/usr/share/backgrounds/epidermis/%s" % self.codeName])
        progressCallback(1.0)
    
    def activate(self, progressCallback=None, shell=None):
        client = gconf.client_get_default()
        if progressCallback is None:
            progressCallback = lambda(a1): False
        if progressCallback(0.0): return
        if os.path.exists(os.path.join("/usr/share/backgrounds/epidermis", self.codeName, self.wallpaperFilename)) and self.wallpaperFilename != "":
            client.set_string("/desktop/gnome/background/picture_filename", \
                os.path.join("/usr/share/backgrounds/epidermis/", self.codeName, self.wallpaperFilename))
        elif self.wallpaperFilename != "":
            client.set_string("/desktop/gnome/background/picture_filename", "")
        if progressCallback(0.5): return
        client.set_string("/desktop/gnome/background/picture_options", self.pictureOptions)
        client.set_string("/desktop/gnome/background/color_shading_type", self.colourShadingType)
        client.set_string("/desktop/gnome/background/primary_color", self.primaryColour)
        if self.secondaryColour != "":
            client.set_string("/desktop/gnome/background/secondary_color", self.secondaryColour)
        progressCallback(1.0)
    
    def read(self, filePath):
        Pigment.read(self, filePath)
        
        from xml.dom import minidom
        doc = minidom.parse(filePath)
        root = doc.getElementsByTagName("pigment")[0]
        
        attachmentPath = filePath[:(-1*len(filePath.split("/")[-1]))]
        self.add_attachment(os.path.join(attachmentPath, "data", pigment.get_first_text( \
            root.getElementsByTagName("filename")[0])))
        
        for xmlName, var in (("pictureOptions", "pictureOptions"), \
        ("colorShadingType", "colourShadingType"), ("primaryColor", "primaryColour")):
            setattr(self, var, pigment.get_first_text(root.getElementsByTagName(xmlName)[0]))
        
        if len(root.getElementsByTagName("secondaryColor")) == 1:
            self.secondaryColour = pigment.get_first_text(root.getElementsByTagName("secondaryColor")[0])
        else:
            self.secondaryColour = ""
        
    def information_xml(self):
        """Returns the XML document describing the pigment"""
        doc = Pigment.information_xml(self)
        root = doc.getElementsByTagName("pigment")[0]
        
        bitsToAdd = [("filename", self.wallpaperFilename), \
        ("pictureOptions", self.pictureOptions), ("colorShadingType", self.colourShadingType), \
        ("primaryColor", self.primaryColour)]
        if self.secondaryColour != "":
            bitsToAdd.append(("secondaryColor", self.secondaryColour))
        
        for xmlName, varr in bitsToAdd:
            xmlBit = doc.createElement(xmlName)
            tn = doc.createTextNode(varr)
            xmlBit.appendChild(tn)
            root.appendChild(xmlBit)
        
        return doc
    
    
    def is_installed(self, installationDirectory=None):
        """Returns whether this pigment is installed
        installationDirectory: the directory in which to search for already 
        installed pigments"""
        if Pigment.is_installed(self, installationDirectory):
            if os.path.exists(os.path.join("/usr/share/backgrounds/epidermis/", self.codeName, self.wallpaperFilename)):
                return True
        return False
    
    def is_activated(self):
        """Returns whether this pigment is applied. To check if the effects
        have been activated regardless of Epidermis' pigments, use is_meat_activated"""
        if self.is_meat_activated():
            client = gconf.engine_get_default()
            if self.pictureOptions != client.get_string("/desktop/gnome/background/picture_options") or \
            self.colourShadingType != client.get_string("/desktop/gnome/background/color_shading_type") or \
            self.primaryColour != client.get_string("/desktop/gnome/background/primary_color"):
                return False
            if self.secondaryColour != "" and self.secondaryColour != client.get_string("/desktop/gnome/background/secondary_color"):
                return False
            return True
        else:
            return False

    def is_attachment_correct(self, attachmentIndex=None):
        if attachmentIndex is None and len(self.attachments) != 1:
            return False
        if not attachmentIndex is None:
            if attachmentIndex >= len(self.attachments):
                print >> sys.stderr, "no attachment with index ", attachmentIndex
            att = self.attachments[attachmentIndex]
        else:
            att = self.attachments[0]
        if not os.path.isfile(att):
            return False
        extension = att.split(".")[-1].lower()
        if not extension in ("png", "jpg", "gif", "svg"):
            return False
        return Pigment.is_attachment_correct(self, attachmentIndex)
    
    def convert_from_meat(self, meat, progressCallback=None):
        # TODO: convert_from_meat
        pass
    
    def is_meat_installed(self):
        """Returns whether the meat of this pigment or similar to it is installed"""
        (md1, md2) = (hashlib.md5(), hashlib.md5())
        try:
            f1 = file(self.wallpaperPath, 'rb')
            f2 = file(os.path.join("/usr/share/backgrounds/epidermis/", self.codeName, self.wallpaperFilename), 'rb')
        except:
            return False
        for mm, ff in ((md1, f1), (md2, f2)):
            ss = ff.read(8096)
            if not ss:
                break
            mm.update(ss)
        return m1 == m2
        
    
    def is_meat_activated(self):
        """Returns whether the meat of this pigment or similar to it is activated"""
        (md1, md2) = (hashlib.md5(), hashlib.md5())
        client = gconf.client_get_default()
        try:
            f1 = file(self.wallpaperPath, 'rb')
            f2 = file(client.get_string("/desktop/gnome/background/picture_filename"), 'rb')
        except:
            return False
        for mm, ff in ((md1, f1), (md2, f2)):
            ss = ff.read(8096)
            if not ss:
                break
            mm.update(ss)
        return m1 == m2
    
    def does_activation_require_root(self):
        """Returns whether the activation requires root rights.
        If it does, the pigment object will need a pexpect bash shell as root
        to be passed to some of its methods"""
        return False
    
    def match_filter(self, string, lang="en"):
        """Returns whether this pigment should be a result for a search for the
        string passed"""
        for ii in (self.get_human_name(lang), self.get_description(lang)):
            if string.lower() in ii.lower():
                return True
        return False
    
    def get_display_data(self, index=0):
        """Returns the object of index to be added to a gtk.ListStore, this
        could be a string, a gtk.gdk.Pixbuf or a custom widget, it should
        be rendered using type.create_cell_renderer"""
        return

def main():
    wp = Wallpaper()
    print wp.information_xml().toxml()
    print wp.previewFilename

if __name__ == "__main__":
    main()